package expr;

public abstract class Expression {

    public abstract double valueAt(double x);

    public abstract Expression derivative();

    public double integral(double min, double max, int n) {
        double sum = 0.;
        double width = (max - min) / n;

        for (int i = 0; i < n - 1; i++) {
            sum += trapezoidArea(width, valueAt(min + i * width), valueAt(min + (i + 1) * width));
        }
        return sum;
    }

    private double trapezoidArea(double h, double a, double b) {
        return h * (a + b) * 0.5;
    }

    @Override
    public String toString() {
        return asString();
    }

    protected abstract String asString();

    protected int getWeight() {
        return Integer.MAX_VALUE;
    }

    public Expression multiply(Expression other) {
        return other.multiplyHelper(this); // double dispatch
    }

    protected Expression multiplyHelper(Expression other) {
        return new Multiply(other, this);
    }

    public Expression add(Expression other) {
        return other.addHelper(this);
    }

    protected Expression addHelper(Expression other) {
        return new Add(other, this);
    }
}
