package machine;

import machine.instructions.Instruction;

public class FunctionDefinition {

    private final String name;
    private final boolean returnsValue;
    private final int numberOfArguments;
    private final Instruction[] instructions;

    public FunctionDefinition(String name, boolean returnsValue, int numberOfArguments, Instruction[] instructions) {
        this.name = name;
        this.returnsValue = returnsValue;
        this.numberOfArguments = numberOfArguments;
        this.instructions = instructions;
    }

    public String getName() {
        return name;
    }

    public void execute(ProgramExecutionState programExecutionState) {
        for (Instruction instruction : instructions) {
            instruction.execute(programExecutionState);
        }
    }

    public boolean validate(Program program, ProgramValidationState validationState) {
        return validateInvocations(program, validationState) && validateStackUsage();
    }

    private boolean validateInvocations(Program program, ProgramValidationState validationState) {
        for (Instruction instruction : instructions) {
            if (!instruction.validate(this, program, validationState)) {
                return false;
            }
        }
        return true;
    }

    private boolean validateStackUsage() {
        int count = numberOfArguments;
        for (Instruction instruction : instructions) {
            count -= instruction.getStackPops();
            if (count < 0) {
                return false;
            }
            count += instruction.getStackPushes();
        }
        return count == (returnsValue ? 1 : 0);
    }

    public boolean isProperInvocation(int numberOfArguments, boolean returnsValue) {
        return this.numberOfArguments == numberOfArguments && this.returnsValue == returnsValue;
    }
}
