package machine;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

public class Program {

    private final FunctionDefinition[] functions;

    public Program(FunctionDefinition[] functions) {
        this.functions = functions;
    }

    public FunctionDefinition getFunction(String name) {
        for (FunctionDefinition function : functions) {
            if (function.getName().equals(name)) {
                return function;
            }
        }
        return null;
    }

    public boolean validate() {
        ProgramValidationState validationState = new ProgramValidationState(functions);
        return validateContainsMain()
                && validateNoFunctionRedefinitions()
                && validateFunctions(validationState)
                && validationState.validateNoRecursion();
    }

    private boolean validateContainsMain() {
        return getFunction("main") != null;
    }

    private boolean validateNoFunctionRedefinitions() {
        // most basic way to do this
        for (int i = 0; i < functions.length; i++) {
            for (int j = i + 1; j < functions.length; j++) {
                if (functions[i].getName().equals(functions[j].getName())) {
                    return false;
                }
            }
        }

        // this is the same as the code above
        Set<String> functionNames = new HashSet<>();
        for (FunctionDefinition function : functions) {
            if (!functionNames.add(function.getName())) {
                return false;
            }
        }

        // this is also the same, but uses java streams
        if (Arrays.stream(functions).map(FunctionDefinition::getName).distinct().count() != functions.length) {
            return false;
        }
        return true;
    }

    private boolean validateFunctions(ProgramValidationState validationState) {
        for (FunctionDefinition function : functions) {
            if (!function.validate(this, validationState)) {
                return false;
            }
        }
        return true;
    }
}
