package machine;

public class ProgramValidationState {

    private final FunctionDefinition[] functions;

    private final boolean[][] invocations;

    public ProgramValidationState(FunctionDefinition[] functions) {
        this.functions = functions;
        this.invocations = new boolean[functions.length][functions.length];
    }

    public int getFunctionIndex(String name) {
        for (int i = 0; i < functions.length; i++) {
            if (functions[i].getName().equals(name)) {
                return i;
            }
        }
        return -1;
    }

    public void setInvocation(String caller, String callee) {
        int callerIndex = getFunctionIndex(caller);
        int calleeIndex = getFunctionIndex(callee);
        invocations[callerIndex][calleeIndex] = true;
    }

    public boolean validateNoRecursion() {
        return validateNoRecursionStep(1, getFunctionIndex("main"));
    }

    private boolean validateNoRecursionStep(int depth, int index) {
        if (depth > functions.length) {
            return false;
        }
        for (int i = 0; i < functions.length; i++) {
            if (invocations[index][i]) {
                if (!validateNoRecursionStep(depth + 1, i)) {
                    return false;
                }
            }
        }
        return true;
    }
}
