package machine.instructions;

import machine.FunctionDefinition;
import machine.Program;
import machine.ProgramExecutionState;
import machine.ProgramValidationState;

public class FunctionInvocation implements Instruction {

    private final String name;
    private final boolean returnsValue;
    private final int numberOfArguments;

    public FunctionInvocation(String name, boolean returnsValue, int numberOfArguments) {
        this.name = name;
        this.returnsValue = returnsValue;
        this.numberOfArguments = numberOfArguments;
    }

    @Override
    public void execute(ProgramExecutionState programExecutionState) {
        programExecutionState.getFunction(name).execute(programExecutionState);
    }

    @Override
    public int getStackPops() {
        return numberOfArguments;
    }

    @Override
    public int getStackPushes() {
        return returnsValue ? 1 : 0;
    }

    @Override
    public boolean validate(FunctionDefinition caller, Program program, ProgramValidationState validationState) {
        FunctionDefinition function = program.getFunction(name);
        if (function == null) {
            return false;
        }
        validationState.setInvocation(caller.getName(), name);
        return function.isProperInvocation(numberOfArguments, returnsValue);
    }
}
